"""
Copyright (c) 2024 Agiflow

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from typing import Collection

from opentelemetry.instrumentation.instrumentor import BaseInstrumentor
from opentelemetry.trace import get_tracer
from wrapt import wrap_function_wrapper
from agiflow.opentelemetry.instrumentation.constants.qdrant import APIS
from agiflow.opentelemetry.instrumentation.utils import (
  method_wrapper,
)

from agiflow.opentelemetry.instrumentation.qdrant.hooks import (
  CollectionCallSpanCapture,
)
from agiflow.version import __version__


class QdrantInstrumentation(BaseInstrumentor):
    """
    The QdrantInstrumentation class represents the Qdrant instrumentation
    """

    def instrumentation_dependencies(self) -> Collection[str]:
        return ["qdrant-client >= 1.9.0"]

    def _instrument(self, **kwargs):
        tracer_provider = kwargs.get("tracer_provider")
        tracer = get_tracer(__name__, __version__, tracer_provider)

        for method, _ in APIS.items():
            wrap_function_wrapper(
                "qdrant_client.qdrant_client",
                f"QdrantClient.{method.lower()}",
                method_wrapper(
                  tracer=tracer,
                  method=method,
                  SpanCapture=CollectionCallSpanCapture
                )
            )

    def _instrument_module(self, module_name):
        pass

    def _uninstrument(self, **kwargs):
        pass
